@@ -25,6 +25,7 @@ import shutil
 import subprocess
 import sys
 import tempfile
+import time
 import warnings
 
 from breezy import ui
@@ -42,9 +43,10 @@ from breezy.transform import revert
 from debian.deb822 import Deb822
 
 
-__version__ = (0, 13, 1)
+__version__ = (0, 16)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'possible', None]
+DEFAULT_MINIMUM_CERTAINTY = 'certain'
 
 
 class NoChanges(Exception):
@@ -54,6 +56,11 @@ class NoChanges(Exception):
 class FixerFailed(Exception):
     """Base class for fixer script failures."""
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+        return self.args == other.args
+
 
 class UnsupportedCertainty(Exception):
     """Unsupported certainty."""
@@ -72,6 +79,14 @@ class FixerScriptFailed(FixerFailed):
                 self.path, self.returncode,
                 self.errors.decode(errors='replace')))
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+        return (
+            self.path == other.path and
+            self.returncode == other.returncode and
+            self.errors == other.errors)
+
 
 class DescriptionMissing(Exception):
     """The fixer script did not provide a description on stdout."""
@@ -84,6 +99,13 @@ class NotDebianPackage(Exception):
         super(NotDebianPackage, self).__init__(tree.basedir)
 
 
+class GeneratedControlFile(Exception):
+    """The specified file is generated."""
+
+    def __init__(self, path):
+        self.path = path
+
+
 class PendingChanges(Exception):
     """The directory has pending changes."""
 
@@ -102,6 +124,7 @@ class FixerResult(object):
 
     def __repr__(self):
         return "%s(%r, fixed_lintian_tags=%r, certainty=%r)" % (
+                self.__class__.__name__,
                 self.description, self.fixed_lintian_tags, self.certainty)
 
     def __eq__(self, other):
@@ -123,14 +146,16 @@ class Fixer(object):
         self.name = name
         self.lintian_tags = lintian_tags
 
-    def run(self, basedir, current_version, compat):
+    def run(self, basedir, current_version, compat_release,
+            trust_package=False):
         """Apply this fixer script.
 
         Args:
           basedir: Directory in which to run
           current_version: The version of the package that is being created or
             updated
-          compat: Compatibility level (a Debian release name)
+          compat_release: Compatibility level (a Debian release name)
+          trust_package: Whether to run code from the package
         Returns:
           A FixerResult object
         """
@@ -147,10 +172,14 @@ class ScriptFixer(Fixer):
     def __repr__(self):
         return "<ScriptFixer(%r)>" % self.name
 
-    def run(self, basedir, current_version, compat_release):
+    def run(self, basedir, current_version, compat_release,
+            minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
+            trust_package=False):
         env = dict(os.environ.items())
         env['CURRENT_VERSION'] = str(current_version)
         env['COMPAT_RELEASE'] = compat_release
+        env['MINIMUM_CERTAINTY'] = minimum_certainty
+        env['TRUST_PACKAGE'] = 'true' if trust_package else 'false'
         with tempfile.SpooledTemporaryFile() as stderr:
             p = subprocess.Popen(self.script_path, cwd=basedir,
                                  stdout=subprocess.PIPE, stderr=stderr,
@@ -211,10 +240,14 @@ def read_desc_file(path):
     dirname = os.path.dirname(path)
     with open(path, 'r') as f:
         for paragraph in Deb822.iter_paragraphs(f):
-            yield ScriptFixer(
-                os.path.splitext(paragraph['Fix-Script'])[0],
-                [tag.strip() for tag in paragraph['Lintian-Tags'].split(',')],
-                os.path.join(dirname, paragraph['Fix-Script']))
+            name = os.path.splitext(paragraph['Fix-Script'])[0]
+            script_path = os.path.join(dirname, paragraph['Fix-Script'])
+            if 'Lintian-Tags' in paragraph:
+                tags = [tag.strip()
+                        for tag in paragraph['Lintian-Tags'].split(',')]
+            else:
+                tags = []
+            yield ScriptFixer(name, tags, script_path)
 
 
 def available_lintian_fixers(fixers_dir=None):
@@ -343,8 +376,37 @@ def only_changes_last_changelog_block(tree):
         return str(new_cl) == str(old_cl)
 
 
+def reset_tree(local_tree):
+    """Reset a tree back to its basis tree.
+
+    This will leave ignored and detritus files alone.
+
+    Args:
+      local_tree: tree to work on
+    """
+    revert(local_tree, local_tree.branch.basis_tree(), None)
+    deletables = list(iter_deletables(
+        local_tree, unknown=True, ignored=False, detritus=False))
+    delete_items(deletables)
+
+
+def certainty_sufficient(actual_certainty, minimum_certainty):
+    """Check if the actual certainty is sufficient.
+
+    Args:
+      actual_certainty: Actual certainty with which changes were made
+      minimum_certainty: Minimum certainty to keep changes
+    Returns:
+      boolean
+    """
+    if actual_certainty == 'possible' and minimum_certainty == 'certain':
+        return False
+    return True
+
+
 def run_lintian_fixer(local_tree, fixer, committer=None,
-                      update_changelog=None, compat_release=None):
+                      update_changelog=None, compat_release=None,
+                      minimum_certainty=None, trust_package=False):
     """Run a lintian fixer on a tree.
 
     Args:
@@ -354,6 +416,9 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
       update_changelog: Whether to add a new entry to the changelog
       compat_release: Minimum release that the package should be usable on
         (e.g. 'stable' or 'unstable')
+      minimum_certainty: How certain the fixer should be
+        about its changes.
+      trust_package: Whether to run code from the package if necessary
     Returns:
       tuple with set of FixerResult, summary of the changes
     """
@@ -373,15 +438,20 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         increment_version(current_version)
     if compat_release is None:
         compat_release = 'sid'
+    if minimum_certainty is None:
+        minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
     try:
-        result = fixer.run(local_tree.basedir, current_version=current_version,
-                           compat_release=compat_release)
+        result = fixer.run(
+            local_tree.basedir, current_version=current_version,
+            compat_release=compat_release,
+            minimum_certainty=minimum_certainty,
+            trust_package=trust_package)
     except BaseException:
-        revert(local_tree, local_tree.branch.basis_tree(), None)
-        deletables = list(iter_deletables(
-            local_tree, unknown=True, ignored=False, detritus=False))
-        delete_items(deletables)
+        reset_tree(local_tree)
         raise
+    if not certainty_sufficient(result.certainty, minimum_certainty):
+        reset_tree(local_tree)
+        raise NoChanges("Script didn't make any changes")
     local_tree.smart_add([local_tree.basedir])
     if local_tree.supports_setting_file_ids():
         RenameMap.guess_renames(
@@ -427,7 +497,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
 
 def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                        verbose=False, committer=None,
-                       compat_release=None):
+                       compat_release=None, minimum_certainty=None,
+                       trust_package=False):
     """Run a set of lintian fixers on a tree.
 
     Args:
@@ -438,32 +509,44 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
       committer: Optional committer (name and email)
       compat_release: Minimum release that the package should be usable on
         (e.g. 'sid' or 'stretch')
+      minimum_certainty: How certain the fixer should be
+        about its changes.
+      trust_package: Whether to run code from the package if necessary
     Returns:
       Tuple with two lists:
-        list of tuples with (lintian-tag, certainty, description) of fixers
-        that ran list of script names for fixers that failed to run
+        1. list of tuples with (lintian-tag, certainty, description) of fixers
+           that ran
+        2. dictionary mapping fixer names for fixers that failed to run to the
+           error that occurred
     """
-    failed_fixers = []
+    if local_tree.has_filename('debian/control.in'):
+        raise GeneratedControlFile('debian/control.in')
+    failed_fixers = {}
     fixers = list(fixers)
     ret = []
     with ui.ui_factory.nested_progress_bar() as pb:
         for i, fixer in enumerate(fixers):
             pb.update('Running fixer %r on %s' % (fixer, local_tree.basedir),
                       i, len(fixers))
+            start = time.time()
             try:
                 result, summary = run_lintian_fixer(
                         local_tree, fixer, update_changelog=update_changelog,
-                        committer=committer, compat_release=compat_release)
+                        committer=committer, compat_release=compat_release,
+                        minimum_certainty=minimum_certainty,
+                        trust_package=trust_package)
             except FixerFailed as e:
-                failed_fixers.append(fixer.name)
+                failed_fixers[fixer.name] = e
                 if verbose:
-                    note('Fixer %r failed to run.', fixer)
+                    note('Fixer %r failed to run.', fixer.name)
                     sys.stderr.write(str(e))
             except NoChanges:
                 if verbose:
-                    note('Fixer %r made no changes.', fixer)
+                    note('Fixer %r made no changes. (took: %.2fs)',
+                         fixer.name, time.time() - start)
             else:
                 if verbose:
-                    note('Fixer %r made changes.', fixer)
+                    note('Fixer %r made changes. (took %.2fs)',
+                         fixer.name, time.time() - start)
                 ret.append((result, summary))
     return ret, failed_fixers
