@@ -51,6 +51,8 @@ from . import (  # noqa: E402
     get_committer,
     run_lintian_fixers,
     version_string,
+    SUPPORTED_CERTAINTIES,
+    DEFAULT_MINIMUM_CERTAINTY,
     )
 
 
@@ -93,6 +95,17 @@ def main(argv=None):
         '--identity',
         help='Print user identity that would be used when committing',
         action='store_true', default=False)
+    # Hide the minimum-certainty option for the moment.
+    parser.add_argument(
+        '--minimum-certainty',
+        type=str,
+        choices=SUPPORTED_CERTAINTIES,
+        default=DEFAULT_MINIMUM_CERTAINTY,
+        help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--trust',
+        action='store_true',
+        help=argparse.SUPPRESS)
     parser.add_argument(
         'fixers', metavar='FIXER', nargs='*',
         help='specific fixer to run')
@@ -150,7 +163,9 @@ def main(argv=None):
                     wt, fixers,
                     update_changelog=args.update_changelog,
                     compat_release=compat_release,
-                    verbose=args.verbose)
+                    verbose=args.verbose,
+                    minimum_certainty=args.minimum_certainty,
+                    trust_package=args.trust)
             except NotDebianPackage:
                 note("%s: Not a debian package.", wt.basedir)
                 return 1
@@ -161,12 +176,16 @@ def main(argv=None):
             all_tags = set()
             for result, summary in applied:
                 all_tags.update(result.fixed_lintian_tags)
-            note("Lintian tags fixed: %r" % all_tags)
+            if all_tags:
+                note("Lintian tags fixed: %r" % all_tags)
+            else:
+                note("Some changes were made, "
+                     "but there are no affected lintian tags.")
         else:
             note("No changes made.")
         if failed and not args.verbose:
             note("Some fixer scripts failed to run: %r. "
-                 "Run with --verbose for details.", set(failed))
+                 "Run with --verbose for details.", set(failed.keys()))
         if args.diff:
             from breezy.diff import show_diff_trees
             show_diff_trees(
