@@ -35,10 +35,12 @@ from lintian_brush import (
     Fixer,
     FixerFailed,
     FixerResult,
+    GeneratedControlFile,
     NoChanges,
     NotDebianPackage,
     PendingChanges,
     available_lintian_fixers,
+    certainty_sufficient,
     get_committer,
     increment_version,
     only_changes_last_changelog_block,
@@ -83,7 +85,8 @@ Lintian-Tags: i-fix-another-tag, no-extension
 
 class DummyFixer(Fixer):
 
-    def run(self, basedir, current_version, compat_release):
+    def run(self, basedir, current_version, compat_release, minimum_certainty,
+            trust_package):
         with open(os.path.join(basedir, 'debian/control'), 'a') as f:
             f.write('a new line\n')
         return FixerResult("Fixed some tag.\nExtended description.",
@@ -92,7 +95,8 @@ class DummyFixer(Fixer):
 
 class FailingFixer(Fixer):
 
-    def run(self, basedir, current_version, compat_release):
+    def run(self, basedir, current_version, compat_release, minimum_certainty,
+            trust_package):
         with open(os.path.join(basedir, 'debian/foo'), 'w') as f:
             f.write("blah")
         with open(os.path.join(basedir, 'debian/control'), 'a') as f:
@@ -211,11 +215,43 @@ Arch: all
                 tree.get_file_lines('debian/control')[-1],
                 b"a new line\n")
 
+    def test_simple_modify_too_uncertain(self):
+        tree = self.make_test_tree()
+
+        class UncertainFixer(Fixer):
+            def run(self, basedir, current_version, compat_release,
+                    minimum_certainty, trust_package):
+                with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
+                    f.write("test")
+                return FixerResult("Renamed a file.", certainty='possible')
+        with tree.lock_write():
+            self.assertRaises(
+                NoChanges, run_lintian_fixer,
+                tree, UncertainFixer('dummy', 'some-tag'),
+                update_changelog=False, minimum_certainty='certain')
+        self.assertEqual(1, tree.branch.revno())
+
+    def test_simple_modify_acceptably_uncertain(self):
+        tree = self.make_test_tree()
+
+        class UncertainFixer(Fixer):
+            def run(self, basedir, current_version, compat_release,
+                    minimum_certainty, trust_package):
+                with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
+                    f.write("test")
+                return FixerResult("Renamed a file.", certainty='possible')
+        with tree.lock_write():
+            result, summary = run_lintian_fixer(
+                tree, UncertainFixer('dummy', 'some-tag'),
+                update_changelog=False, minimum_certainty='possible')
+        self.assertEqual(2, tree.branch.revno())
+
     def test_new_file(self):
         tree = self.make_test_tree()
 
         class NewFileFixer(Fixer):
-            def run(self, basedir, current_version, compat_release):
+            def run(self, basedir, current_version, compat_release,
+                    minimum_certainty, trust_package):
                 with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
                     f.write("test")
                 return FixerResult("Created new file.", ['some-tag'])
@@ -245,7 +281,8 @@ Arch: all
         tree = self.make_test_tree()
 
         class RenameFileFixer(Fixer):
-            def run(self, basedir, current_version, compat_release):
+            def run(self, basedir, current_version, compat_release,
+                    minimum_certainty, trust_package):
                 os.rename(os.path.join(basedir, 'debian/control'),
                           os.path.join(basedir, 'debian/control.blah'))
                 return FixerResult("Renamed a file.")
@@ -273,7 +310,8 @@ Arch: all
         tree = self.make_test_tree()
 
         class EmptyFixer(Fixer):
-            def run(self, basedir, current_version, compat_release):
+            def run(self, basedir, current_version, compat_release,
+                    minimum_certainty, trust_package):
                 return FixerResult("I didn't actually change anything.")
         with tree.lock_write():
             self.assertRaises(
@@ -322,7 +360,7 @@ Arch: all
                     self.tree, [FailingFixer('fail', 'some-tag')],
                     update_changelog=False)
         self.assertEqual([], applied)
-        self.assertEqual(['fail'], failed)
+        self.assertEqual({'fail': FixerFailed('Not successful')}, failed)
         with self.tree.lock_read():
             self.assertEqual(
                 [], list(self.tree.iter_changes(self.tree.basis_tree())))
@@ -347,12 +385,25 @@ Arch: all
                               ['some-tag'], 'certain'),
                   'Fixed some tag.')],
                 applied)
-        self.assertEqual([], failed)
+        self.assertEqual({}, failed)
         self.assertEqual(2, self.tree.branch.revno())
         self.assertEqual(
                 self.tree.get_file_lines('debian/control')[-1],
                 b"a new line\n")
 
+    def test_generated_control_file(self):
+        with self.tree.lock_write():
+            self.build_tree_contents([
+                ('debian/control.in', """\
+Source: blah
+""")])
+            self.tree.add(['debian/control.in'])
+            self.tree.commit('Add control.in')
+            self.assertRaises(
+                GeneratedControlFile, run_lintian_fixers,
+                self.tree, [DummyFixer('dummy', 'some-tag')],
+                update_changelog=False)
+
 
 class HonorsVcsCommitter(TestCaseWithTransport):
 
@@ -624,3 +675,15 @@ class GetCommitterTests(TestCaseWithTransport):
         self.assertEqual(
             'Yet Another Committer <yet@example.com>',
             get_committer(tree))
+
+
+class CertaintySufficientTests(TestCase):
+
+    def test_sufficient(self):
+        self.assertTrue(certainty_sufficient('certain', 'certain'))
+        self.assertTrue(certainty_sufficient('certain', 'possible'))
+        self.assertTrue(certainty_sufficient('certain', None))
+        self.assertTrue(certainty_sufficient('possible', None))
+
+    def test_insufficient(self):
+        self.assertFalse(certainty_sufficient('possible', 'certain'))
